local dotgrid = {}

function dotgrid:shuffle(tbl)
	for i = #tbl, 2, -1 do
		local j = math.random(i)
		tbl[i], tbl[j] = tbl[j], tbl[i]
	end
	return tbl
end

function dotgrid:new()
	local dots = wibox.widget.base.make_widget();

	dots.on_color = {1, 0, 0}
	dots.off_color = {0.5, 0, 0}
	dots.value = 0
	dots.max = 100
	dots.rows = 5
	dots.cols = 5
	dots.spacing = 2

	dots.map = {}
	local idx
	for i=1, dots.rows*dots.cols do
		dots.map[i] = false;
	end
	--dots.map = dotgrid:shuffle(dots.map)


	function dots:fit(context, width, height)
		local m = math.min(width, height)
		return m, m
	end

	function dots:draw(context, cr, width, height)
		local col_size = (height - self.spacing * (self.rows-1)) // self.rows
		local row_size = (width - self.spacing * (self.cols-1)) // self.cols

		-- Compute rounding offsets for better centering
		local xoffset = (
			width - (
				(col_size * self.cols) +
				(self.spacing * (self.cols - 1))
			)
		) // 2

		local yoffset = (
			height - (
				(row_size * self.rows) +
				(self.spacing * (self.rows - 1))
			)
		) // 2

		-- How many dots should be on
		local on = math.floor((self.value / self.max) * (self.rows * self.cols))

		-- How many dote are currently on
		local current_value = 0
		for i=1, dots.rows*dots.cols do
			if dots.map[i] then
				current_value = current_value + 1
			end
		end

		local change, remove
		if current_value ~= on then
			change = on - current_value
			remove = change < 0
			change = math.abs(change)

			local flip
			for i=1, change do
				if remove then
					-- Flip the nth on dot
					flip = math.random(1, current_value)
					for i=1, dots.rows*dots.cols do
						if dots.map[i] then
							if flip == 1 then
								dots.map[i] = false
								break
							else 
								flip = flip - 1
							end
						end
					end
					current_value = current_value - 1
				else
					-- Flip the nth off dot
					flip = math.random(1, dots.rows*dots.cols - current_value)
					for i=1, dots.rows*dots.cols do
						if not dots.map[i] then
							if flip == 1 then
								dots.map[i] = true
								break
							else 
								flip = flip - 1
							end
						end
					end
					current_value = current_value + 1
				end
			end

		end

		for r=1, self.rows do
			for c=1, self.cols do
				idx = c + ((r-1)*self.cols)

				--idx_mapped = self.map[idx] - 1
				--r_mapped = (idx_mapped % self.rows) + 1
				--c_mapped = (idx_mapped // self.cols) + 1

				cr:new_path();
				if self.map[idx] then
					cr:set_source_rgb(self.on_color[1], self.on_color[2], self.on_color[3])
				else
					cr:set_source_rgb(self.off_color[1], self.off_color[2], self.off_color[3])
				end

				cr:rectangle(
					(c-1) * (col_size + self.spacing) + xoffset,
					(r-1) * (row_size + self.spacing) + yoffset,
					col_size,
					row_size
				)
				cr:fill()
			end
		end

		
	end

	return dots
end

return dotgrid